/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_topic.c,v 1.12.4.4 2002/03/09 17:06:24 lloydy Exp $
 *
 */
#include "modules.h"

CMIRCD_MODULE(1.02,CMD_CLIENT | CMD_SERVER);

/* Utility routines */
static int topic_client(ClientInfo ,Cmd );
static int topic_server(ServerInfo ,Cmd );
static int topic_inform(ClientInfo ,ServerInfo ,ChannelInfo );
static int topic_nb_inform(char *,ServerInfo ,ChannelInfo );

/*
* change the topic of a channel
*/
int cmd_topic(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd)
{
	
	if(c_ptr) {
		topic_client(c_ptr,cmd);
	} else {
		if(s_ptr) {
			topic_server(s_ptr,cmd);
		}
	}

	 
	return 1;

}

/*
 * Handle a topic change from a client
 */
static int topic_client(ClientInfo c_ptr,Cmd cmd)
{
	ChannelMember w_ptr=NULL;
	ChannelInfo chan=NULL;
	char buffer[BUF_SIZE];

	/* Check the params */
	if(!(check_params(cmd,1))) {
		return 0;
	}

	/* Check to see if the channel exists */
	chan=locate_channel(cmd->params[0]);
	if(!chan) {
		ircd_sprintf(buffer,BUF_SIZE,"%s :No such channel",cmd->params[0]);
		send_numeric(c_ptr,get_my_server(),403,buffer);
		return 0;
	}

	/* Check to see if they're on the channel */
	w_ptr=channel_is_client_on(c_ptr,chan);
	if(!w_ptr) {
		ircd_sprintf(buffer,BUF_SIZE,"%s :You're not on that channel",chan->name);
		send_numeric(c_ptr,get_my_server(),442,buffer);
		return 0;
	}

	/* What should we do ? */
	if(cmd->text!=NULL || cmd->had_colon==1) {
		/* Change the topic */
		if( (chan->mode & CMODE_TOPIC) && 
			!(w_ptr->chan_mode & CHANUSR_OP || w_ptr->chan_mode & CHANUSR_HALFOP) ) {
			/* They can't change it */
			ircd_sprintf(buffer,BUF_SIZE,"%s :Ha! You're no chan op",chan->name);
			send_numeric(c_ptr,get_my_server(),482,buffer);
		} else {
			/* They can change it */
			strncpy(chan->topic_set,c_ptr->nick,NICK_LENGTH);
			chan->topic_ts=time(NULL);
			if(cmd->text) {
				strncpy(chan->topic,cmd->text,TOPIC_LENGTH);
			} else {
				chan->topic[0]='\0';
			}
			topic_inform(c_ptr,NULL,chan);
		}
			
	} else {
		/* Report it */
		if(chan->topic[0]=='\0') {
			/* No Topic Set */
			ircd_sprintf(buffer,BUF_SIZE,"%s :No topic is set",chan->name);
			send_numeric(c_ptr,get_my_server(),331,buffer);
		} else {
			/* Report the topic */
			ircd_sprintf(buffer,BUF_SIZE,"%s :%s",chan->name,chan->topic);
			send_numeric(c_ptr,get_my_server(),332,buffer);
			/* Report the timestamp, and the person who changed it */
			ircd_sprintf(buffer,BUF_SIZE,"%s %s %ld",chan->name,chan->topic_set,chan->topic_ts);
			send_numeric(c_ptr,get_my_server(),333,buffer);	
		}
	}

	return 1;
}

/*
 * A remote server change
 * or a netburst
 */
static int topic_server(ServerInfo s_ptr,
			Cmd cmd)
{
	ClientInfo c_ptr=NULL;
	ChannelInfo chan=NULL;
	time_t remote_ts;

	chan=locate_channel(cmd->params[0]);
	c_ptr=client_locate(cmd->params[1]);
	remote_ts=strtoul(cmd->params[2],NULL,10);

	if( !(chan) || !(c_ptr) ) 
		return 0;
	
	/* Are we in a netburst */
	if(lc_strncmp(cmd->origin,cmd->params[1],-1)==0) {
		/* This isn't a netburst */
		strncpy(chan->topic_set,c_ptr->nick,NICK_LENGTH);
		chan->topic_ts=remote_ts;
		if(!cmd->text) {
			chan->topic[0]='\0';
		} else {
			strncpy(chan->topic,cmd->text,TOPIC_LENGTH);
		}

		topic_inform(c_ptr,s_ptr,chan);
	} else {
		if(remote_ts < chan->topic_ts) {
			/* This is a netburst */
			if(cmd->params[1][0]=='*') {
				chan->topic_set[0]='\0';
			} else {				
				strncpy(chan->topic_set,cmd->params[1],NICK_LENGTH);
			}
			chan->topic_ts=remote_ts;
			if(!cmd->text) {
				chan->topic[0]='\0';
			} else {
				strncpy(chan->topic,cmd->text,TOPIC_LENGTH);
			}
			topic_nb_inform(cmd->origin,s_ptr,chan);
		}
	}

	return 1;
}
	
	

/*
 * Inform the local clients
 * and tell the other servers of the change
 */
static int topic_inform(ClientInfo c_ptr,
		        ServerInfo s_ptr,
			ChannelInfo chan)
{
	char buffer[BUF_SIZE];

	/* Tell the local clients */
	ircd_sprintf(buffer,BUF_SIZE,"TOPIC %s :%s",chan->name,chan->topic);
	if(c_ptr->fd!=-1) 
		send_client_client(c_ptr,c_ptr,buffer);
	send_to_local_channel(c_ptr,chan,buffer);

	/* Send it to the servers */
	ircd_sprintf(buffer,BUF_SIZE,"TOPIC %s %s %ld :%s",chan->name,chan->topic_set,
							 chan->topic_ts,chan->topic);
	servers_relay(s_ptr,c_ptr->nick,buffer);

	return 1;
}

/*
 * Has to be slightly different for a netburst
 */
static int topic_nb_inform(char *origin,
			   ServerInfo s_ptr,
			   ChannelInfo chan)
{
	char buffer[BUF_SIZE];
	
	/* Tell the local clients */
	ircd_sprintf(buffer,BUF_SIZE,":%s TOPIC %s :%s",origin,chan->name,chan->topic);
	send_to_local_channel(NULL,chan,buffer);

	/* Send it to the servers */
	ircd_sprintf(buffer,BUF_SIZE,"TOPIC %s %s %ld :%s",chan->name,chan->topic_set,
						   chan->topic_ts,chan->topic);
	servers_relay(s_ptr,origin,buffer);

	return 1;
}

